(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROGRAMM  IEPS_AGP                               06.06.94  *)
(*                                                                          *)
(*  (c) Markus Neher    Institute for Applied Mathematics                   *)
(*                      D-76128 Karlsruhe University, Germany               *)
(*                      e-mail: ae16@rz.uni-karlsruhe.de                    *)
(*                                                                          *)
(*  INVERSES EIGENWERTPROBLEM :                                             *)
(*                                                                          *)
(*  BERECHNUNG EINES SYMMETRISCHEN KUBISCHEN SPLINE-POTENTIALS              *)
(*  ZU VORGEGEBENEN EIGENWERTEN                                             *)
(*                                                                          *)
(*  AUSGANGSPROBLEM :      - U" + P(X)  U(X) =   U(X)                    *)
(*                                                                          *)
(*  HIER : VOLLSTNDIGES REKONSTRUKTIONSVERFAHREN AUS DER DISSERTATION      *)
(*  EINSCHLIESSUNGSSCHRITT STARTET MIT ALEFELD-GIENGER-POTRA-NHERUNG       *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

PROGRAM IEPS_AGP( INPUT , EIN , OUTPUT , AUS );

USE i_ari , mv_ari , mvi_ari , lss , ilss , seig , seig_app;
USE dep , service , splines , iep_tayk , iep_rawa , iep_iawa;

CONST M =  7;            (*  ANZAHL DER STTZSTELLEN DES SPLINES  *)
      N =  3;            (*  N = M DIV 2 = ANZAHL DER EW - 1      *)
      P = 15;            (*  ORDNUNG DER TAYLORENTWICKLUNG VON U  *)

LABEL 99;

VAR EW , I , J , COUNT , ERR            : INTEGER;
    TEVA , ST                           : INTEGER;
    DIVI , DIVI_2                       : INTEGER;
    DIVI_3 , DIVI_4 , DIVI_5 , PI       : REAL;
    MOLLIFIER , HNORM , ETA , ETA_ALT   : REAL;
    H , T , VR_U2                       : REAL;
    WIDTH_ALT , WIDTH_AKT               : REAL;
    WIDTH_H , X_H , FCP_H               : REAL;
    IX_H                                : INTERVAL;
    NCL , NCL_MID , AGP                 : BOOLEAN;
    IDIVI_2 , IDIVI_3 , IDIVI_4         : INTERVAL;
    IDIVI_5 , IPI , IH , IHI , IT       : INTERVAL;
    EIG_ZIEL , EIG_AKT , FCP_AKT , HFCP : RVECTOR[0..N];
    ZUWACHS_MID                         : RVECTOR[0..N];
    ZUWACHS                             : IVECTOR[0..N];
    FUNKT_MAT                           : RMATRIX[ 0..N , 0..N ];

    DIVISOR                             : ARRAY[0..N] OF INTEGER;
    VRU                                 : RVECTOR[1..2];
    VR_U2Q                              : RVECTOR[0..N];
    SPP0 , SPP                          : RSPLINE[ 0..3 , 0..M ];
    SPQ0 , SPQ                          : RSVECTOR[ 0..N , 0..3 , 0..M ];
    TRVECP                              : RTVEC[ 0..300 , 0..P ];
    TRMATQ                              : RTMAT[ 0..N , 0..300 , 0..P ];

    VIU                                 : IVECTOR[1..2];
    VI_U2                               : INTERVAL;
    VI_U2Q                              : IVECTOR[0..N];
    IFCP_AKT , IFCP_ALT                 : IVECTOR[0..N];
    IEIG_ZIEL , IEIG_AKT , IEIG_AKT_MID : IVECTOR[0..N];
    ILAST_FUNKT_MAT , IFUNKT_MAT        : IMATRIX[ 0..N , 0..N ];
    ISPP0 , ISPP , ISPP0_MID , ISPP_MID : ISPLINE[ 0..3 , 0..M ];
    ISPPH                               : ISPLINE[ 0..3 , 0..M ];
    ISPQ0 , ISPQ                        : ISVECTOR[ 0..N , 0..3 , 0..M ];
    TIVECP , TIVECP_RST                 : ITVEC[ 0..3000 , 0..P ];
    TIVECP_MID , TIVECP_RST_MID         : ITVEC[ 0..3000 , 0..P ];
    TIMATQ                              : ITMAT[ 0..N , 0..3000 , 0..P ];
    FUNKT_NAME                          : STRING;
    EIN , DATEIN , AUS , AUS2           : TEXT;


(*     EW  : NR DES EIGENWERTS                                    *)
(*     VR_U2 = U^2  , VR_U2Q[I] = Q[I] * U^2                      *)

(*--------------------------------------------------------------------------*)

PROCEDURE WRITE_HEAD( HEADLINE : STRING );

(*  KOPFZEILE DRUCKEN  *)
   
BEGIN
   WRITELN( AUS , 'Rekonstruktion der Funktion' );
   WRITELN( AUS );
   WRITELN( AUS , FUNKT_NAME );
   WRITELN( AUS );
   WRITELN( AUS );
   WRITELN( AUS , HEADLINE );
   WRITELN( AUS );
   WRITELN( AUS );
END;

(*--------------------------------------------------------------------------*)

FUNCTION TAYKFCSP( T : REAL; SPP : RSPLINE ) : RTKO[0..P];

(*  BERECHNET TAYLORKOEFFIZIENTEN DES SPLINES P(X)  *)

VAR I   : INTEGER;
    TRP : RTKO[0..P];
    
BEGIN (*  IN TRP[N] STEHT DIE N-TE ABLEITUNG DURCH N!  *)

   TRP[0] := ( SPP[3,TEVA] * T / 3 + SPP[2,TEVA] ) * T / 2 + SPP[1,TEVA];
   TRP[0] := TRP[0] * T + SPP[0,TEVA];
   TRP[1] := ( SPP[3,TEVA] * T / 2 + SPP[2,TEVA] ) * T + SPP[1,TEVA];
   TRP[2] := ( SPP[3,TEVA] * T  + SPP[2,TEVA] ) / 2;
   TRP[3] := SPP[3,TEVA] / 6;

   FOR I := 4 TO P DO
       TRP[I] := 0;

   TAYKFCSP := TRP;

END;   

(*--------------------------------------------------------------------------*)

FUNCTION TAYKFCSP( IT : INTERVAL; SPP : ISPLINE ) : ITKO[0..P];

(*  BERECHNET TAYLORKOEFFIZIENTEN DES SPLINES P(X)  *)

VAR I   : INTEGER;
    TIP : ITKO[0..P];
    
BEGIN (*  IN TIP[N] STEHT DIE N-TE ABLEITUNG DURCH N!  *)

   TIP[0] := ( SPP[3,TEVA] * IT / 3 + SPP[2,TEVA] ) * IT / 2 + SPP[1,TEVA];
   TIP[0] := TIP[0] * IT + SPP[0,TEVA];
   TIP[1] := ( SPP[3,TEVA] * IT / 2 + SPP[2,TEVA] ) * IT + SPP[1,TEVA];
   TIP[2] := ( SPP[3,TEVA] * IT  + SPP[2,TEVA] ) / 2;
   TIP[3] := SPP[3,TEVA] / 6;

   FOR I := 4 TO P DO
       TIP[I] := 0;

   TAYKFCSP := TIP;

END;   

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  HAUPTPROGRAMM                                                           *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

BEGIN RESET( EIN );
   REWRITE( AUS );

   (*  VARIABLEN VORBESETZEN  *)

   PI:= 3.141592653589793;
   IPI:= INTVAL( RVAL('3.141592653589793',-1) , RVAL('3.141592653589794',1) );
   ETA := 1;

   (* GLOBALE SCHRITTWEITE SETZEN *)

   H   := PI / M;
   IH  := IPI / M;
   IHI := INTVAL( 0 , IH.SUP );

   (*  FUNKTIONSNAME EINLESEN  *)

   READLN;
   READLN;
   READLN( FUNKT_NAME );
   READLN;

   (*  KOPFZEILE DRUCKEN  *)
   
   WRITE_HEAD( 'Einschlieung einer Lsung mit AGP-Startintervall' );
   WRITELN( AUS );
   WRITELN( AUS , 'Ziel-Eigenwerte :' );
   WRITELN( AUS );
   WRITELN( AUS );

   (*  EIGENWERTE UND NHERUNGEN DER FUNKTIONSWERTE  *)
   (*  IN DEN SIGNIFIKANTEN STTZSTELLEN EINLESEN    *)
    
   FOR I := 0 TO N DO BEGIN
       READLN( IEIG_ZIEL[I] );
       EIG_ZIEL[I] := MID( IEIG_ZIEL[I] );
       IF IEIG_ZIEL[I].INF = IEIG_ZIEL[I].SUP THEN
          WRITELN( AUS , I : 1 , '. Eigenwert : ' , EIG_ZIEL[I] )
       ELSE BEGIN
          WRITE(    AUS , I : 1 , '. Eigenwert : ' );
          IWRITELN( AUS , IEIG_ZIEL[I] );
       END;
       READLN( EIN , FCP_AKT[I] );
       SPP[0,I] := FCP_AKT[I];
   END;

   WRITELN( AUS );
   WRITELN( AUS );
   WRITELN(AUS,'Differenzen der Intervall-Mittelpunkte der Ziel-Eigenwerte :');
   WRITELN( AUS );
   WRITELN( AUS );
   FOR I := 1 TO N DO BEGIN
       WRITE(   AUS , I : 1 , '. Eigenwertdifferenz : ' );
       WRITELN( AUS , EIG_ZIEL[I] - EIG_ZIEL[I-1] );
   END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  SCHRITT 1 : BERECHNUNG EINES GROBEN NHERUNGSSPLINES MIT DEM            *)
(*              RITZ-VERFAHREN                                              *)
(*                                                                          *)
(*  SCHNELLSTMGLICHER ALGORITHMUS: ES WIRD VORAUSGESETZT, DASS ZU DEN      *)
(*                                  BASISSPLINES MATRIZEN BEREITSTEHEN,     *)
(*  AUS DENEN DIE RITZ-MATRIX DES AKTUELLEN SPLINES DURCH LINEARKOMBI-      *)
(*  NATIONEN UND ERGNZUNG DER DIAGONALE BERECHNET WERDEN KANN              *)
(*                                                                          *)
(*  ==> KEINE LSUNG VON DIFFERENTIALGLEICHUNGEN, KEINE EINSCHLIESSUNGEN    *)
(*                                                                          *)
(*  IN JEDEM ITERATIONSSCHRITT DES NEWTON-VERFAHRENS IST DANN ALSO NUR      *)
(*  DAS EIGENWERTPROBLEM FR DIE RITZMATRIX ZU LSEN UND DAS LGS FR DIE    *)
(*  NEWTON-ITERATION                                                        *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

   REWRITE( AUS2 , 'ews_ritz.aus' );

   (*  SPLINE BERECHNEN  *)
    
   SPLINE( PI , SPP );

   WRITEPG( AUS );
   WRITE_HEAD( 'Ritz-Verfahren' );
   WRITELN( AUS , 'Startnherung des Splines :' );
   WRITELN( AUS );
   SWRITE(  AUS , 'P' , PI , SPP );
      
   COUNT := 0;
   
   REPEAT  (*  "ENDLOS"-SCHLEIFE  *)

      COUNT := COUNT + 1;
      
      (*EIGENWERTE UND FUNKTIONALMATRIX DES SPLINES FR 30 ANSATZFUNKTIONEN  *)
      (*BEIM RITZ-VERFAHREN MIT DER PROZEDUR DEPS_RITZ IM MODUL DEP BERECHNEN*)

      WRITELN( FUNKT_NAME );
      FOR I := 1 TO 8 DO BEGIN
          WRITE( 'Prozedur DEPS_RITZ berechnet Eigenwertnherungen bei der' );
          WRITELN( ' Ritz-Iteration...' , COUNT : 2 );
      END;

      DEPS_RITZ( SPP , M , N , 30 , EIG_AKT , FUNKT_MAT );

      (*  EIGENWERTE UND FUNKTIONALMATRIX AUSGEBEN  *)
       
      WRITEPG( AUS );
      WRITE_HEAD( 'Ritz-Verfahren' );
      IF COUNT = 1 THEN BEGIN
         WRITE( AUS , 'Eigenwerte der Startn' );
         WRITE( AUS2 , 'Funktionalmatrix der Startn' );
         END
      ELSE BEGIN
         WRITE( AUS , 'Eigenwerte der ' );
         WRITE( AUS , COUNT - 1 : 1 , '. N' );
         WRITE( AUS2 , 'Funktionalmatrix der ' );
         WRITE( AUS2 , COUNT - 1 : 1 , '. N' );
      END;
      WRITELN( AUS , 'herung :' );
      WRITELN( AUS );
      WRITELN( AUS2 , 'herung :' );
      WRITELN( AUS2 );
      FOR I := 0 TO N DO
          WRITELN( AUS , I : 1 , '. Eigenwert : ' , EIG_AKT[I] );
      WRITELN( AUS );
      FOR I := 1 TO N DO BEGIN
          WRITE(   AUS , I : 1 , '. Eigenwertdifferenz : ' );
          WRITELN( AUS , EIG_AKT[I] - EIG_AKT[I-1] );
      END;
      WRITELN( AUS );
      MRWRITE( AUS2 , 'DF' , FUNKT_MAT );
      
      (*  NCHSTE ITERIERTE DES SPLINES BERECHNEN  *)
      
      WRITELN( FUNKT_NAME );
      FOR I := 1 TO 8 DO BEGIN
          WRITE( 'Prozedur LSS lst Newton-Schritt bei der Ritz-Iteration.' );
          WRITELN( '..................' , COUNT : 2 );
      END;

      LSS( FUNKT_MAT , EIG_ZIEL - EIG_AKT , ZUWACHS , ERR );
      EIG_AKT := EIG_ZIEL;
      ZUWACHS_MID := MID( ZUWACHS );
      FCP_AKT := FCP_AKT + ZUWACHS_MID;

      (*  HILFSGRSSEN FR DEN EINSCHLIESSUNGSSCHRITT BERECHNEN  *)

      ETA_ALT := ETA;
      ETA := MAX_NORM( ZUWACHS_MID );
                              
      WRITELN( AUS );
      WRITELN( AUS , 'Zuwachs  q^(', COUNT:1, ') - q(', COUNT-1:1, ') : ' );
      WRITELN( AUS );
      VRWRITE( AUS , 'ZW' , ZUWACHS_MID );
      WRITELN( AUS );
      WRITELN( AUS );
      WRITELN( AUS , 'eta_' , COUNT : 1 , ' = Maxnorm( Zuwachs ) : ' , ETA );
      WRITELN( AUS );
      WRITE(   AUS , '8 * eta_' , COUNT : 1 , '^3 / ' );
      WRITE(   AUS , '( Maxnorm( q^(' , COUNT : 1 , ') ) * eta_' );
      WRITE(   AUS , COUNT - 1 : 1 , '^2 ) : ' );
      WRITELN( AUS , 8 * ( ETA**3 ) / ( MAX_NORM(FCP_AKT) * SQR(ETA_ALT) ) );

(*    MOLLIFIER := MAX_NORM( MID( ZUWACHS ) );                              *)
(*    IF MOLLIFIER < 2 THEN                                                 *)
(*       FCP_AKT := FCP_AKT + MID( ZUWACHS )                                *)
(*    ELSE                                                                  *)
(*       FCP_AKT := FCP_AKT + 2 * MID( ZUWACHS ) / MOLLIFIER;               *)
       
      FOR I := 0 TO N DO
          SPP[0,I] := FCP_AKT[I];
           
      SPLINE( PI , SPP );

      WRITEPG( AUS );
      WRITE_HEAD( 'Ritz-Verfahren' );
      WRITELN( AUS , COUNT : 1 , '. Nherung des Splines :' );
      WRITELN( AUS );
      SWRITE(  AUS , 'P' , PI , SPP );

   UNTIL ( MAX_NORM( MID ( ZUWACHS ) ) < 1.0E-04 * MAX_NORM( FCP_AKT ) )
         OR ( COUNT = 10 );

   IF COUNT = 10 THEN BEGIN

      (*  ABBRUCH MIT VERMUTETER DIVERGENZ  *)

      WRITELN( AUS );
      WRITELN( AUS );
      WRITE(   AUS , 'Abbruch wegen berschreitung der Hchstzahl ' );
      WRITELN( AUS , 'an Iterationen.' );
      WRITELN( AUS );
      WRITELN( AUS , 'Das Verfahren scheint zu divergieren.' );
      WRITEPG( AUS );

      (*  SPRUNG ZUM PROGRAMMENDE  *)

      GOTO 99;

   END;
(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  SCHRITT 2 : BERECHNUNG EINES GUTEN NHERUNGSSPLINES MIT DEM             *)
(*              VERFAHREN VON LOHNER                                        *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME NHERUNGSWEISE GELST                               *)
(*                                                                          *)
(*  AUSGANGSPROBLEM :      - U" + P(X)  U(X) =   U(X)                    *)
(*                                                                          *)
(*  TRANSFORMIERT AUF :      U1' = U2                                       *)
(*                           U2' = ( P(X) -  )  U1                        *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

   REWRITE( AUS2 , 'ews_app.aus' );

   (*  HEURISTISCH OPTIMALEN DIVISOR BERECHNEN        *)
   (*  DIVISOREN STRECKEN DAS ZEITINTERVALL DER DGL,  *)
   (*  STEUERN ALSO INDIREKT DIE SCHRITTWEITE         *)

   FOR EW := 0 TO N DO BEGIN
       FOR I := 0 TO N DO
           HFCP[I] := FCP_AKT[I] - EIG_ZIEL[EW];
       HNORM := SQRT( MAX_NORM( HFCP ) );
       DIVISOR[EW] := 1 + ROUND( ( 6 * HNORM + 15 ) / M );

       IF DIVISOR[EW] * M > 300 THEN BEGIN

          (*  SPEICHERPLATZBEDINGTER ABBRUCH  *)

          WRITEPG( AUS );
          WRITE_HEAD( 'Anfangswertprobleme nherungsweise gelst' );
          WRITELN( AUS );
          WRITE(   AUS , 'Es sind zu viele Sttzstellen, nmlich ' );
          WRITELN( AUS , M * DIVISOR[EW] : 1 , ' statt 300.' );
          GOTO 99;

       END;

   END;

   (*  SPLINE BERECHNEN  *)

   FOR I := 0 TO N DO
       SPP0[0,I] := FCP_AKT[I];
    
   SPLINE( PI , SPP0 );
   SPP := SPP0;

   (*  BASISSPLINES BERECHNEN  *)

   FOR EW := 0 TO N DO BEGIN
       FOR I := 0 TO N DO            
           SPQ0[EW,0,I] := 0;
       SPQ0[EW,0,EW] := 1;   
       SPLINE( PI , SPQ0[EW] );
   END;
   SPQ := SPQ0;

   WRITEPG( AUS );
   WRITE_HEAD( 'Anfangswertprobleme nherungsweise gelst' );
   WRITE(   AUS , 'Intervall I = [ 0 ,  ]        ' );
   WRITELN( AUS , 'Taylor-Ordnung P = ' , P : 1 );
   WRITELN( AUS );
   WRITELN( AUS , 'Schrittweiten :' );
   WRITELN( AUS );
   FOR EW := 0 TO N DO
       WRITELN( AUS , EW:1 , '. Eigenwert :  h =  / ' , M * DIVISOR[EW]:1 );
   WRITELN( AUS );
   WRITELN( AUS );
   WRITELN( AUS , 'Startnherung des Splines :' );
   WRITELN( AUS );
   SWRITE(  AUS , 'P' , PI , SPP0 );
      
   COUNT := 0;
   
   REPEAT  (*  "ENDLOS"-SCHLEIFE  *)

      COUNT := COUNT + 1;
      
      WRITEPG( AUS2 );
      WRITELN( AUS2 , 'Anfangswertprobleme nherungsweise gelst' );
      WRITELN( AUS2 );

      FOR EW := 0 TO N DO BEGIN

          DIVI   := DIVISOR[EW];
          DIVI_2 := SQR( DIVI );
          DIVI_3 := DIVI ** 3;
          DIVI_4 := SQR( DIVI_2 );
          DIVI_5 := DIVI_2 * DIVI_3;

          (*  TAYLORKOEFFIZIENTEN DER POTENTIALFUNKTION BEREITSTELLEN  *)

          IF ( EW = 0 ) OR ( DIVISOR[EW] <> DIVISOR[EW-1] ) THEN BEGIN
             
             FOR I := 0 TO M DO BEGIN
                 SPP[0,I] := SPP0[0,I] / DIVI_2;
                 SPP[1,I] := SPP0[1,I] / DIVI_3;
                 SPP[2,I] := SPP0[2,I] / DIVI_4;
                 SPP[3,I] := SPP0[3,I] / DIVI_5;
                 FOR J := 0 TO N DO BEGIN
                     SPQ[J,0,I] := SPQ0[J,0,I];
                     SPQ[J,1,I] := SPQ0[J,1,I] / DIVI;
                     SPQ[J,2,I] := SPQ0[J,2,I] / DIVI_2;
                     SPQ[J,3,I] := SPQ0[J,3,I] / DIVI_3;
                 END;
             END;
              
             FOR TEVA := 0 TO M - 1 DO
                 FOR ST := 0 TO DIVI - 1 DO BEGIN
                     T := H * ST;
                     TRVECP[ DIVI * TEVA + ST ] := TAYKFCSP( T , SPP );  
                     FOR J := 0 TO N DO
                         TRMATQ[ J, DIVI*TEVA + ST ] := TAYKFCSP( T, SPQ[J] );
                 END;

          END;

          (*  VORITERATION : FUNKTIONSWERTE AN DER STELLE "SPP" BERECHNEN  *)

          (*  PROZEDUR EW_APP IM MODUL DEP BERECHNET EIGENWERTNHERUNG  *)
          (*  BIS ZUR MAXIMAL DURCH LSEN VON ANFANGSWERTPROBLEMEN AUF  *)
          (*  DEM RECHNER ERZIELBAREN GENAUIGKEIT (=STILLSTAND DER EW-  *)
          (*  ITERATION DURCH DAS SEKANTENVERFAHREN)                    *)

          EIG_AKT[EW] := EW_APP( M, P, EW, DIVI, TRVECP, EIG_ZIEL[EW], AUS2 );

          (*  ZUR BERECHNUNG DER FUNKTIONALMATRIX  *)
          (*  SIND ANFANGSWERTPROBLEME ZU LSEN    *)

          AWA_NWT( M * DIVI , P , H , EIG_AKT[EW] / DIVI_2 ,
                   TRVECP , TRMATQ , VRU , VR_U2 , VR_U2Q );

          WRITELN( AUS2 , ' = ' , EIG_AKT[EW] , ' :' );
          WRITELN( AUS2 );
          WRITELN( AUS2 , 'U  ( ' , DIVI : 2 , ' ) = ' , VRU[1] );
          WRITELN( AUS2 , 'U'' ( ' , DIVI : 2 , ' ) = ' , VRU[2] );
          WRITELN( AUS2 , 'U( ' , DIVI : 2 , ' ) = ' , VR_U2 );

          FOR I := 0 TO N DO
              WRITELN( AUS2 , 'Q[' , I : 1 , '] * U = ' , VR_U2Q[I] );
          WRITELN( AUS2 );

          (*  EW-TE ZEILE DER FUNKTIONALMATRIX BERECHNEN                 *)

          FOR I := 0 TO N DO
              FUNKT_MAT[EW,I] := VR_U2Q[I] / VR_U2;

      END; (* FOR , EW *)

      (*  FUNKTIONALMATRIX AUSGEBEN  *)
       
      WRITEPG( AUS2 );
      WRITELN( AUS2 , 'Anfangswertprobleme nherungsweise gelst' );
      WRITELN( AUS2 );
      IF COUNT = 1 THEN 
         WRITE( AUS2 , 'Funktionalmatrix der Startn' )
      ELSE
         WRITE( AUS2 , 'Funktionalmatrix der ' , COUNT - 1 : 1 , '. N' );
      WRITELN( AUS2 , 'herung :' );
      WRITELN( AUS2 );
      MRWRITE( AUS2 , 'DF' , FUNKT_MAT );

      (*  NCHSTE ITERIERTE DES SPLINES BERECHNEN  *)
      
      WRITELN( FUNKT_NAME );
      FOR I := 1 TO 8 DO BEGIN
          WRITE( 'Prozedur LSS lst Newton-Schritt bei der DGL-Iteration..' );
          WRITELN( '..................' , COUNT : 2 );
      END;

      LSS( FUNKT_MAT , EIG_ZIEL - EIG_AKT , ZUWACHS , ERR );
      EIG_AKT := EIG_ZIEL;
      ZUWACHS_MID := MID( ZUWACHS );
      FCP_AKT := FCP_AKT + ZUWACHS_MID;

      (*  HILFSGRSSEN FR DEN EINSCHLIESSUNGSSCHRITT BERECHNEN  *)

      ETA_ALT := ETA;
      ETA := MAX_NORM( ZUWACHS_MID );

      WRITEPG( AUS );
      WRITE_HEAD( 'Anfangswertprobleme nherungsweise gelst' );
      WRITELN( AUS , 'Zuwachs  q^(', COUNT:1, ') - q(', COUNT-1:1, ') : ' );
      WRITELN( AUS );
      VRWRITE( AUS , 'ZW' , ZUWACHS_MID );
      WRITELN( AUS );
      WRITELN( AUS );
      WRITELN( AUS , 'eta_' , COUNT : 1 , ' = Maxnorm( Zuwachs ) : ' , ETA );
      WRITELN( AUS );
      WRITE(   AUS , '8 * eta_' , COUNT : 1 , '^3 / ' );
      WRITE(   AUS , '( Maxnorm( q^(' , COUNT : 1 , ') ) * eta_' );
      WRITE(   AUS , COUNT - 1 : 1 , '^2 ) : ' );
      WRITELN( AUS , 8 * ( ETA**3 ) / ( MAX_NORM(FCP_AKT) * SQR(ETA_ALT) ) );

(*    MOLLIFIER := MAX_NORM( MID( ZUWACHS ) );                              *)
(*    IF MOLLIFIER < 2 THEN                                                 *)
(*       FCP_AKT := FCP_AKT + MID( ZUWACHS )                                *)
(*    ELSE                                                                  *)
(*       FCP_AKT := FCP_AKT + 2 * MID( ZUWACHS ) / MOLLIFIER;               *)
       
      FOR I := 0 TO N DO
          SPP0[0,I] := FCP_AKT[I];
           
      SPLINE( PI , SPP0 );

      WRITEPG( AUS );
      WRITE_HEAD( 'Anfangswertprobleme nherungsweise gelst' );
      WRITELN( AUS , COUNT : 1 , '. Nherung des Splines :' );
      WRITELN( AUS );
      SWRITE(  AUS , 'P' , PI , SPP0 );

(* UNTIL MAX_NORM( MID ( ZUWACHS ) ) < 1.0E-12 * MAX_NORM( FCP_AKT ); *)
   UNTIL 8 * ( ETA ** 3 ) / ( MAX_NORM( FCP_AKT ) * SQR( ETA_ALT ) ) < 1.0E-14;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  SCHRITT 3 : BERECHNUNG EINES EINSCHLIESSENDEN INTERVALLSPLINES          *)
(*              MIT DEM VERFAHREN VON LOHNER                                *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT INTERVALLEINSCHLIESSUNG GELST                  *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

   REWRITE( AUS2 , 'ews_bds.aus' );

   (*  SPLINE BESETZEN  *)
    
   FOR I := 0 TO N DO
       ISPP0_MID[0,I] := FCP_AKT[I];

   (*  LETZTE FUNKTIONALMATRIX BESETZEN  *)

   FOR J := 0 TO N DO
       FOR I := 0 TO N DO
           ILAST_FUNKT_MAT[I,J] := FUNKT_MAT[I,J];

   (*  SPLINE BERECHNEN  *)
    
   SPLINE( IPI , ISPP0_MID );

   (*  BASISSPLINES BERECHNEN  *)

   FOR EW := 0 TO N DO BEGIN
       FOR I := 0 TO N DO            
           ISPQ0[EW,0,I] := 0;
       ISPQ0[EW,0,EW] := 1;   
       SPLINE( IPI , ISPQ0[EW] );
   END;
   ISPQ := ISPQ0;

   WRITEPG( AUS );
   WRITE_HEAD( 'Einschlieungsschritt' );
   WRITE(   AUS , 'Intervall I = [ 0 ,  ]        ' );
   WRITELN( AUS , 'Taylor-Ordnung P = ' , P : 1 );
   WRITELN( AUS );
   WRITELN( AUS , 'Schrittweiten :' );
   WRITELN( AUS );
   FOR EW := 0 TO N DO
       WRITELN( AUS , EW:1 , '. Eigenwert :  h =  / ' , M * DIVISOR[EW]:1 );
   WRITELN( AUS );
   WRITELN( AUS );
      
   COUNT := 0;
   
   REPEAT  (*  "ENDLOS"-SCHLEIFE  *)

      COUNT := COUNT + 1;
      
      FOR EW := 0 TO N DO BEGIN

          DIVI    := 4 * DIVISOR[EW] + 4;

          IF DIVI * M > 3000 THEN BEGIN

             (*  SPEICHERPLATZBEDINGTER ABBRUCH  *)

             WRITEPG( AUS );
             WRITE_HEAD( 'Anfangswertprobleme intervallmig gelst' );
             WRITELN( AUS );
             WRITE(   AUS , 'Es sind zu viele Sttzstellen, nmlich ' );
             WRITELN( AUS , M * DIVI : 1 , ' statt 3000.' );
             GOTO 99;

          END;

          IDIVI_2 := SQR( DIVI );
          IDIVI_3 := DIVI ** 3;
          IDIVI_4 := SQR( IDIVI_2 );
          IDIVI_5 := IDIVI_2 * IDIVI_3;

          (*  TAYLORKOEFFIZIENTEN DER POTENTIALFUNKTION BEREITSTELLEN  *)

          IF ( EW = 0 ) OR ( DIVISOR[EW] <> DIVISOR[EW-1] ) THEN BEGIN
             
             FOR I := 0 TO M DO BEGIN
                 ISPP_MID[0,I] := ISPP0_MID[0,I] / IDIVI_2;
                 ISPP_MID[1,I] := ISPP0_MID[1,I] / IDIVI_3;
                 ISPP_MID[2,I] := ISPP0_MID[2,I] / IDIVI_4;
                 ISPP_MID[3,I] := ISPP0_MID[3,I] / IDIVI_5;
             END;
              
             FOR TEVA := 0 TO M - 1 DO
                 FOR ST := 0 TO DIVI - 1 DO BEGIN
                     IT := IH * ST;
                     TIVECP_MID[ DIVI*TEVA + ST ] := TAYKFCSP( IT , ISPP_MID );
                     TIVECP_RST_MID[DIVI*TEVA+ST] := TAYKFCSP(IT+IHI,ISPP_MID);
                 END;

          END;

          (*  FUNKTIONSWERTE AN DER STELLE "ISPP" BERECHNEN  *)

          (*  PROZEDUR  EWS_BDS  IM MODUL DEP BERECHNET EIGENWERTSCHRANKEN  *)
          (*  GENAUIGKEITS-PARAMETER 0.98 = MAXIMALE GENAUIGKEIT            *)

          IEIG_AKT_MID[EW] := EIG_ZIEL[EW];
          
          IEIG_AKT_MID[EW] := EWS_BDS( M , P , EW , DIVI , TIVECP_MID ,
                              TIVECP_RST_MID , IEIG_AKT_MID[EW], 0.98, AUS2 );

      END; (* FOR , EW *)
      
      (*  JETZT INTERVALLSPLINE GENERIEREN, IN DEM DIE LSUNG VERMUTET WIRD  *)
      (*  VERFAHREN VON ALEFELD-GIENGER-POTRA VERWENDEN                      *)

      (*  ZUM VERGLEICH ZUERST INNERE MINDESTEINSCHLIESSUNG BERECHNEN !!! *)

      WRITELN( FUNKT_NAME );
      FOR I := 1 TO 8 DO BEGIN
          WRITE( 'Prozedur LSS berechnet Mindesteinschlieung.............' );
          WRITELN( '..................' , COUNT : 2 );
      END;

      LSS( ILAST_FUNKT_MAT , IEIG_ZIEL - IEIG_AKT_MID , ZUWACHS , ERR );

      WIDTH_AKT := ZUWACHS[0].SUP -> ZUWACHS[0].INF;
    
      FOR I := 1 TO N DO
          IF WIDTH_AKT < ZUWACHS[I].SUP -> ZUWACHS[I].INF THEN
             WIDTH_AKT := ZUWACHS[I].SUP -> ZUWACHS[I].INF;

      WRITELN( AUS , 'Durchmesser der Mindesteinschlieung : ' , WIDTH_AKT );
      WRITELN( AUS );
      WRITELN( AUS , 'Durchmesser der AGP-Einschlieung    : ' , 2 * ETA );
      AGP := TRUE;
      IF WIDTH_AKT > 2 * ETA THEN BEGIN
         WRITELN( AUS );
         WRITE(   AUS , 'Der Durchmesser der AGP-Einschlieung ' );
         WRITELN( AUS , 'ist zu klein!' );
         WRITELN( AUS );
         WRITE(   AUS , 'Die AGP-Einschlieung wird mit (eta_k * eta_k-1)' );
         WRITELN( AUS , ' durchgefhrt.' );
         ETA := SQRT( ETA * ETA_ALT );
         WRITELN( AUS );
         WRITE(   AUS , 'Durchmesser der vergrberten AGP-Einschlieung : ' );
         WRITELN( AUS , 2 * ETA );
         IF WIDTH_AKT > 2 * ETA THEN BEGIN
            WRITELN( AUS );
            WRITE( AUS,'Der Durchmesser der vergrberten AGP-Einschlieung ');
            WRITELN( AUS , 'ist auch zu klein!' );
            WRITELN( AUS );
            WRITE(   AUS , 'Die Einschlieung wird mit hunderfacher ' );
            WRITELN( AUS , 'Mindesteinschlieung durchgefhrt.' );
            AGP := FALSE;
            ETA := 50 * WIDTH_AKT;
            WRITELN( AUS );
            WRITE(AUS,'Durchmesser der hundertfachen Mindesteinschlieung : ');
            WRITELN( AUS , 2 * ETA );
         END;
      END
      ELSE IF WIDTH_AKT > 0.4 * ETA THEN BEGIN (*  AGP <= 5 * MINDEST  *)
         WRITELN( AUS );
         WRITE(   AUS , 'Die AGP-Einschlieung ist nur ' );
         WRITELN( AUS , 'wenig grer als die Mindesteinschlieung' );
         WRITELN( AUS );
         WRITE(   AUS , 'Die AGP-Einschlieung wird mit (eta_k * eta_k-1)' );
         WRITELN( AUS , ' durchgefhrt.' );
         ETA := SQRT( ETA * ETA_ALT );
         WRITELN( AUS );
         WRITE(   AUS , 'Durchmesser der vergrberten AGP-Einschlieung : ' );
         WRITELN( AUS , 2 * ETA );
      END;

      (*  JETZT INTERVALLSPLINE GENERIEREN, IN DEM DIE LSUNG VERMUTET WIRD  *)

      FOR I := 0 TO N DO BEGIN
          IFCP_AKT[I] := ISPP0_MID[0,I] + ETA * INTVAL( -1 , 1 );
          ISPP0[0,I] := IFCP_AKT[I];
      END;

      WIDTH_AKT := 2 * ETA;
    
      SPLINE( IPI , ISPP0 );

      WRITEPG( AUS );
      IF AGP THEN
         WRITE_HEAD( 'AGP-Einschlieungsschritt' )
      ELSE
   WRITE_HEAD('Einschlieungsschritt mit hundertfacher Mindesteinschlieung');
      WRITELN( AUS , 'Start-Intervalleinschlieung :' );
      WRITELN( AUS );
      SWRITE(  AUS , 'P' , IPI , ISPP0 );

      WRITEPG( AUS );
      IF AGP THEN
         WRITE_HEAD( 'AGP-Einschlieungsschritt' )
      ELSE
   WRITE_HEAD('Einschlieungsschritt mit hundertfacher Mindesteinschlieung');
      WRITELN( AUS2 );
      IF AGP THEN
         WRITELN( AUS2 , 'AGP-Einschlieungsschritt' )
      ELSE
WRITELN(AUS2,'Einschlieungsschritt mit hundertfacher Mindesteinschlieung');
      WRITELN( AUS2 );

      (*  EIGENWERTSCHRANKEN DES INTERVALLSPLINES MIT DEM MONOTONIEPRINZIP  *)
      (*  AUS DEN SCHRANKEN DES MITTELPUNKTSPLINES BERECHNEN                *)

      FOR I := 0 TO 3 DO
          FOR J := 0 TO M DO
              ISPPH[I,J] := ISPP0[I,J] - ISPP0_MID[I,J];

      (*  MAXIMALEN DURCHMESSER DURCH MAXIMAL MGLICHEN DURCHMESSER  *)
      (*  IN DER MITTE ZWISCHEN ZWEI STTZSTELLEN ABSCHTZEN         *)

      WIDTH_H := 0;
      IX_H := PI / ( 2 * M );
      X_H := IX_H.SUP;

      FOR I := 0 TO M - 1 DO BEGIN

          (*  VON X_I NACH RECHTS BIS ZUR MITTE  *)

          FCP_H := SUP( ABS( ISPPH[3,I] ) ) /> 6;
          FCP_H := FCP_H *> X_H +> SUP( ABS( ISPPH[2,I] / 2 ) );
          FCP_H := FCP_H *> X_H +> SUP( ABS( ISPPH[1,I] ) );
          FCP_H := FCP_H *> X_H +> SUP( ABS( ISPPH[0,I] ) );

          IF FCP_H > WIDTH_H THEN
             WIDTH_H := FCP_H;

          (*  VON X_I+1 NACH LINKS BIS ZUR MITTE                  *)
          (*  LINKSSEITIGE DRITTE ABLEITUNG: INDEX I STATT I+1 !  *)

          FCP_H := SUP( ABS( ISPPH[3,I] ) ) /> 6;
          FCP_H := FCP_H *> X_H +> SUP( ABS( ISPPH[2,I+1] / 2 ) );
          FCP_H := FCP_H *> X_H +> SUP( ABS( ISPPH[1,I+1] ) );
          FCP_H := FCP_H *> X_H +> SUP( ABS( ISPPH[0,I+1] ) );

          IF FCP_H > WIDTH_H THEN
             WIDTH_H := FCP_H;

      END;

      (*  FUNKTIONALMATRIX BERECHNEN  *)
      
      FOR EW := 0 TO N DO BEGIN

          IEIG_AKT[EW] := IEIG_AKT_MID[EW] + INTVAL( -WIDTH_H , WIDTH_H );

          DIVI   := DIVISOR[EW];
          IDIVI_2 := SQR( DIVI );
          IDIVI_3 := DIVI ** 3;
          IDIVI_4 := SQR( IDIVI_2 );
          IDIVI_5 := IDIVI_2 * IDIVI_3;

          (*  TAYLORKOEFFIZIENTEN DER POTENTIALFUNKTION BEREITSTELLEN  *)

          IF ( EW = 0 ) OR ( DIVISOR[EW] <> DIVISOR[EW-1] ) THEN BEGIN
             
             FOR I := 0 TO M DO BEGIN
                 ISPP[0,I] := ISPP0[0,I] / IDIVI_2;
                 ISPP[1,I] := ISPP0[1,I] / IDIVI_3;
                 ISPP[2,I] := ISPP0[2,I] / IDIVI_4;
                 ISPP[3,I] := ISPP0[3,I] / IDIVI_5;
                 FOR J := 0 TO N DO BEGIN
                     ISPQ[J,0,I] := ISPQ0[J,0,I];
                     ISPQ[J,1,I] := ISPQ0[J,1,I] / DIVI;
                     ISPQ[J,2,I] := ISPQ0[J,2,I] / IDIVI_2;
                     ISPQ[J,3,I] := ISPQ0[J,3,I] / IDIVI_3;
                 END;
             END;
              
             FOR TEVA := 0 TO M - 1 DO
                 FOR ST := 0 TO DIVI - 1 DO BEGIN
                     IT := IH * ST;
                     TIVECP[ DIVI * TEVA + ST ] := TAYKFCSP( IT , ISPP );
                     TIVECP_RST[ DIVI*TEVA + ST ] := TAYKFCSP( IT+IHI , ISPP );
                     FOR J := 0 TO N DO
                        TIMATQ[ J, DIVI*TEVA + ST ] := TAYKFCSP( IT, ISPQ[J] );
                 END;

          END;

          (*  ZUR BERECHNUNG DER FUNKTIONALMATRIX  *)
          (*  SIND ANFANGSWERTPROBLEME ZU LSEN    *)

          AWA_SNWT( M , DIVI , P , IH , IEIG_AKT[EW] / IDIVI_2 ,
                    TIVECP , TIVECP_RST , TIMATQ , 
                    VIU , VI_U2 , VI_U2Q );

          WRITELN(  AUS , 'Einschlieungen des ' , EW : 1 , '. Eigenwerts :' );
          WRITELN(  AUS );
          WRITE(    AUS , 'Mittelpunktsnherung : ' );
          IWRITELN( AUS , IEIG_AKT_MID[EW] );
          WRITE(    AUS , 'Intervallspline      : ' );
          IWRITELN( AUS , IEIG_AKT[EW] );
          WRITELN(  AUS );
          WRITE(    AUS2 , 'Werte der ' , EW : 1 , '. "Eigenfunktion" '  );
          WRITELN(  AUS2 , 'des Intervallsplines :'  );
          WRITELN(  AUS2 );
          WRITE(    AUS2 , 'U  ( ' , DIVI : 2 , ' ) = ' );
          IWRITELN( AUS2 , VIU[1] );
          WRITE(    AUS2 , 'U'' ( ' , DIVI : 2 , ' ) = ' );
          IWRITELN( AUS2 , VIU[2] );
          WRITE(    AUS2 , ' U       = ' );
          IWRITELN( AUS2 , VI_U2);

          FOR I := 0 TO N DO BEGIN
              WRITE( AUS2 , 'Q[' , I : 1 , '] * U = ' );
              IWRITELN( AUS2 , VI_U2Q[I] );
          END; 
          WRITELN( AUS2 );

          (*  EW-TE ZEILE DER FUNKTIONALMATRIX BERECHNEN                 *)

          FOR I := 0 TO N DO
              IFUNKT_MAT[EW,I] := VI_U2Q[I] / VI_U2;

      END; (* FOR , EW *)

      (*  FUNKTIONALMATRIX AUSGEBEN  *)
       
      WRITEPG( AUS2 );
      WRITELN( AUS2 );
      IF AGP THEN
         WRITELN( AUS2 , 'AGP-Einschlieungsschritt' )
      ELSE
WRITELN(AUS2,'Einschlieungsschritt mit hundertfacher Mindesteinschlieung');
      WRITELN( AUS2 );
      IF COUNT = 1 THEN 
         WRITE( AUS2 , 'Funktionalmatrix der Startn' )
      ELSE
         WRITE( AUS2 , 'Funktionalmatrix der ' , COUNT - 1 : 1 , '. N' );
      WRITELN( AUS2 , 'herung :' );
      WRITELN( AUS2 );
      MIWRITE( AUS2 , 'DF' , IFUNKT_MAT );
      
      (*  NCHSTE ITERIERTE DES SPLINES BERECHNEN  *)
      
      WRITELN( FUNKT_NAME );
      FOR I := 1 TO 8 DO BEGIN
          WRITE( 'Prozedur LSS lst Newton-Schritt bei der Verifizierung' );
          WRITELN( '....................' , COUNT : 2 );
      END;

      LSS( IFUNKT_MAT , IEIG_ZIEL - IEIG_AKT_MID , ZUWACHS , ERR );

      IFCP_ALT := IFCP_AKT;

      FOR I := 0 TO N DO BEGIN
          IFCP_AKT[I] := ISPP0_MID[0,I] + ZUWACHS[I];
          ISPP0[0,I] := IFCP_AKT[I];
      END;
           
      SPLINE( IPI , ISPP0 );

      WRITEPG( AUS );
      IF AGP THEN
         WRITE_HEAD( 'AGP-Einschlieungsschritt' )
      ELSE
   WRITE_HEAD('Einschlieungsschritt mit hundertfacher Mindesteinschlieung');
      WRITELN( AUS , COUNT : 1 , '. Nherung des Splines :' );
      WRITELN( AUS );
      SWRITE(  AUS , 'P' , IPI , ISPP0 );

      WIDTH_ALT := 0;
      WIDTH_AKT := 0;
      NCL := TRUE;
      NCL_MID := TRUE;
    
      FOR I := 0 TO N DO BEGIN
     
         IF ( IFCP_ALT[I].SUP -> IFCP_ALT[I].INF > WIDTH_ALT ) THEN
            WIDTH_ALT := IFCP_ALT[I].SUP -> IFCP_ALT[I].INF;

         IF ( IFCP_AKT[I].SUP -> IFCP_AKT[I].INF > WIDTH_AKT ) THEN
            WIDTH_AKT := IFCP_AKT[I].SUP -> IFCP_AKT[I].INF;

          IF NOT ( IFCP_AKT[I] IN IFCP_ALT[I] ) THEN
             NCL := FALSE;

          IF NOT ( ISPP0_MID[0,I] IN IFCP_AKT[I] ) THEN
             NCL_MID := FALSE;

      END;
   
      WRITELN( AUS );
      WRITELN( AUS );
      WRITELN( AUS );
      IF AGP THEN
         WRITE( AUS , 'Eine verifizierte AGP-Einschlieung konnte ' )
      ELSE
         WRITE( AUS , 'Eine verifizierte Einschlieung konnte ' );
      IF NCL THEN
         WRITELN( AUS , 'berechnet werden.' )
      ELSE
         WRITELN( AUS , 'nicht berechnet werden.' );
      WRITELN( AUS );
      WRITE( AUS , 'Die alte (Punkt-) Nherungslsung ist ' );
      IF NCL_MID THEN
         WRITELN( AUS , 'noch in der neuen Einschlieung enthalten.' )
      ELSE BEGIN
         WRITELN( AUS , 'in der neuen Einschlieung nicht mehr' );
      WRITELN( AUS , 'enthalten.' );
      END;
      WRITELN( AUS );
      WRITELN( AUS , 'Maximaler Durchmesser von [q(x)]_ALT : ' , WIDTH_ALT );
      WRITELN( AUS );
      WRITELN( AUS , 'Eigenwert-Durchmesser von [q(x)]_ALT : ' , 2 * WIDTH_H );
      WRITELN( AUS );
      WRITELN( AUS , 'Maximaler Durchmesser von [q(x)]_NEU : ' , WIDTH_AKT );

(* UNTIL NCL;*)
   UNTIL TRUE;    (*  DERZEITIG ABBRECHEN NACH EINEM EINSCHLIESSUNGSVERSUCH *)

   WRITEPG( AUS );

99: END.

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROGRAMM  IEPS_AGP                                         *)
(*                                                                          *)
(*  NEWTON-VERFAHREN ZUR BERECHNUNG EINES SYMMETRISCHEN KUBISCHEN           *)
(*  SPLINE-POTENTIALS ZU VORGEGEBENEN EIGENWERTEN                           *)
(*                                                                          *)
(*  VOLLSTNDIGES REKONSTRUKTIONSVERFAHREN AUS DER DISSERTATION             *)
(*  EINSCHLIESSUNGSSCHRITT STARTET MIT ALEFELD-GIENGER-POTRA-NHERUNG       *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)
