(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROGRAMM  IEPT_AGP                               06.06.94  *)
(*                                                                          *)
(*  (c) Markus Neher    Institute for Applied Mathematics                   *)
(*                      D-76128 Karlsruhe University, Germany               *)
(*                      e-mail: ae16@rz.uni-karlsruhe.de                    *)
(*                                                                          *)
(*  INVERSES EIGENWERTPROBLEM :                                             *)
(*                                                                          *)
(*  BERECHNUNG EINES SYMMETRISCHEN TRIGONOMETRISCHEN POTENTIALS             *)
(*  ZU VORGEGEBENEN EIGENWERTEN                                             *)
(*                                                                          *)
(*  AUSGANGSPROBLEM :      - U" + P(X)  U(X) =   U(X)                    *)
(*                                                                          *)
(*  HIER : VOLLSTNDIGES REKONSTRUKTIONSVERFAHREN AUS DER DISSERTATION      *)
(*  EINSCHLIESSUNGSSCHRITT STARTET MIT ALEFELD-GIENGER-POTRA-NHERUNG       *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

PROGRAM IEPT_AGP( INPUT , EIN , OUTPUT , AUS );

USE i_ari , mv_ari , mvi_ari , lss , ilss , seig_app;
USE dept , service , iep_tayk , iep_rawa , iep_iawa;

CONST M = 10;       (*  ANZAHL DER STTZSTELLEN ZUR SCHRITTWEITENSTEUERUNG  *)
      N =  5;       (*  N = M DIV 2 = ANZAHL DER EW - 1      *)
      P = 15;       (*  ORDNUNG DER TAYLORENTWICKLUNG VON U  *)

LABEL 99;

VAR EW , I , J , COUNT , ERR , ST       : INTEGER;
    DIVI , DIVI_2                       : INTEGER;
    HNORM , ETA , ETA_ALT               : REAL;
    PI , H , T , VR_U2                  : REAL;
    WIDTH_ALT , WIDTH_AKT               : REAL;
    WIDTH_H , X_H , FCP_H               : REAL;
    IX_H                                : INTERVAL;
    NCL , NCL_MID , AGP                 : BOOLEAN;
    IPI , IH , IHI , IT                 : INTERVAL;
    EIG_ZIEL , EIG_AKT , FCP_AKT        : RVECTOR[0..N];
    FC_Q                                : RMATRIX[ 0..N , 0..N ];
    ZUWACHS_MID                         : RVECTOR[0..N];
    ZUWACHS                             : IVECTOR[0..N];
    FUNKT_MAT                           : RMATRIX[ 0..N , 0..N ];
    DIVISOR                             : ARRAY[0..N] OF INTEGER;
    VRU                                 : RVECTOR[1..2];
    VR_U2Q                              : RVECTOR[0..N];
    TRVECP                              : RTVEC[ 0..700 , 0..P ];
    TRMATQ                              : RTMAT[ 0..N , 0..700 , 0..P ];

    VIU                                 : IVECTOR[1..2];
    VI_U2                               : INTERVAL;
    VI_U2Q                              : IVECTOR[0..N];
    IFCP_AKT , IFCP_ALT , IFCP_MID      : IVECTOR[0..N];
    IEIG_ZIEL , IEIG_AKT , IEIG_AKT_MID : IVECTOR[0..N];
    IFC_Q                               : IMATRIX[ 0..N , 0..N ];
    ILAST_FUNKT_MAT , IFUNKT_MAT        : IMATRIX[ 0..N , 0..N ];
    TIVECP , TIVECP_RST                 : ITVEC[ 0..700 , 0..P ];
    TIVECP_MID , TIVECP_RST_MID         : ITVEC[ 0..700 , 0..P ];
    TIMATQ                              : ITMAT[ 0..N , 0..700 , 0..P ];
    FUNKT_NAME                          : STRING;
    EIN , DATEIN , AUS , AUS2           : TEXT;

(*     EW  : NR DES EIGENWERTS                                    *)
(*     VR_U2 = U^2  , VR_U2Q[I] = Q[I] * U^2                      *)

(*--------------------------------------------------------------------------*)

PROCEDURE WRITE_HEAD( HEADLINE : STRING );

(*  KOPFZEILE DRUCKEN  *)
   
BEGIN
   WRITELN( AUS , 'Rekonstruktion der Funktion' );
   WRITELN( AUS );
   WRITELN( AUS , FUNKT_NAME );
   WRITELN( AUS );
   WRITELN( AUS );
   WRITELN( AUS , HEADLINE );
   WRITELN( AUS );
   WRITELN( AUS );
END;

(*--------------------------------------------------------------------------*)

FUNCTION TAYKFCPT( T : REAL; FCP_AKT : RVECTOR ) : RTKO[0..P];

(*  BERECHNET DIE TAYLORKOEFFIZIENTEN DER     *)
(*  TRIGONOMETRISCHEN POTENTIALFUNKTION P(X)  *)

VAR I   : INTEGER;
    TRP : RTKO[0..P];

BEGIN
   TRP := TKO( P , FCP_AKT[0] );
   FOR I := 1 TO N DO
       TRP := TRP + FCP_AKT[I] * COSKO( P , T , 2 * I / DIVI );

   FOR I := 0 TO P DO
       TRP[I] := TRP[I] / DIVI_2;

   TAYKFCPT := TRP;
   
END;

(*--------------------------------------------------------------------------*)

FUNCTION TAYKFCPT( IT : INTERVAL; IFCP_AKT : IVECTOR ) : ITKO[0..P];

(*  BERECHNET DIE TAYLORKOEFFIZIENTEN DER     *)
(*  TRIGONOMETRISCHEN POTENTIALFUNKTION P(X)  *)

VAR I   : INTEGER;
    TIP : ITKO[0..P];

BEGIN
   TIP := TKO( P , IFCP_AKT[0] );
   FOR I := 1 TO N DO
       TIP := TIP + IFCP_AKT[I] * COSKO( P , IT , 2 * I / INTVAL( DIVI ) );

   FOR I := 0 TO P DO
       TIP[I] := TIP[I] / DIVI_2;

   TAYKFCPT := TIP;
   
END;

(*--------------------------------------------------------------------------*)

FUNCTION RESTGLIED( IT : INTERVAL; IFCP_AKT : IVECTOR ) : INTERVAL;

(*  BERECHNET DEN P-TEN TAYLORKOEFFIZIENTEN DER  *)
(*  TRIGONOMETRISCHEN POTENTIALFUNKTION P(X)     *)

VAR I   : INTEGER;
    TIP : ITKO[0..P];

BEGIN
   TIP := TKO( P , IFCP_AKT[0] );
   FOR I := 1 TO N DO
       TIP := TIP + IFCP_AKT[I] * COSKO( P , IT , 2 * I / INTVAL( DIVI ) );

   RESTGLIED := TIP[P];  (*  HIER KEINE DIVISION DURCH DIVI_2!  *)
   
END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  HAUPTPROGRAMM                                                           *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

BEGIN RESET( EIN );
   REWRITE( AUS );

   (*  VARIABLEN VORBESETZEN  *)

   PI:= 3.141592653589793;
   IPI:= INTVAL( RVAL('3.141592653589793',-1) , RVAL('3.141592653589794',1) );
   ETA := 1;

   (* GLOBALE SCHRITTWEITE SETZEN *)

   H   := PI / M;
   IH  := IPI / M;
   IHI := INTVAL( 0 , IH.SUP );

   (*  FUNKTIONSNAME EINLESEN  *)

   READLN;
   READLN;
   READLN( FUNKT_NAME );
   READLN;

   (*  KOPFZEILE DRUCKEN  *)
   
   WRITE_HEAD( 'Einschlieung einer Lsung mit AGP-Startintervall' );
   WRITELN( AUS );
   WRITELN( AUS , 'Ziel-Eigenwerte :' );
   WRITELN( AUS );
   WRITELN( AUS );

   (*  EIGENWERTE UND NHERUNGEN DER FUNKTIONSWERTE  *)
   (*  IN DEN SIGNIFIKANTEN STTZSTELLEN EINLESEN    *)
    
   FOR I := 0 TO N DO BEGIN
       READLN( IEIG_ZIEL[I] );
       EIG_ZIEL[I] := MID( IEIG_ZIEL[I] );
       IF IEIG_ZIEL[I].INF = IEIG_ZIEL[I].SUP THEN
          WRITELN( AUS , I : 1 , '. Eigenwert : ' , EIG_ZIEL[I] )
       ELSE BEGIN
            WRITE(    AUS , I : 1 , '. Eigenwert : ' );
            IWRITELN( AUS , IEIG_ZIEL[I] );
       END;
       READLN( EIN , FCP_AKT[I] );
   END;

   WRITELN( AUS );
   WRITELN( AUS );
   WRITELN(AUS,'Differenzen der Intervall-Mittelpunkte der Ziel-Eigenwerte :');
   WRITELN( AUS );
   WRITELN( AUS );
   FOR I := 1 TO N DO BEGIN
       WRITE(   AUS , I : 1 , '. Eigenwertdifferenz : ' );
       WRITELN( AUS , EIG_ZIEL[I] - EIG_ZIEL[I-1] );
   END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  SCHRITT 1 : BERECHNUNG EINES GROBEN NHERUNGSPOTENTIALS MIT DEM         *)
(*              RITZ-VERFAHREN                                              *)
(*                                                                          *)
(*  SCHNELLSTMGLICHER ALGORITHMUS: ES WIRD VORAUSGESETZT, DASS ZU DEN      *)
(*                                  TRIGONOMETRISCHEN BASISFUNKTIONEN       *)
(*  MATRIZEN BEREITSTEHEN, AUS DENEN DIE RITZ-MATRIX DES AKTUELLEN          *)
(*  TRIGONOMETRISCHEN POTENTIALS DURCH LINEARKOMBINATIONEN UND ERGNZUNG    *)
(*  DER DIAGONALE BERECHNET WERDEN KANN                                     *)
(*                                                                          *)
(*  ==> KEINE LSUNG VON DIFFERENTIALGLEICHUNGEN, KEINE EINSCHLIESSUNGEN    *)
(*                                                                          *)
(*  IN JEDEM ITERATIONSSCHRITT DES NEWTON-VERFAHRENS IST DANN ALSO NUR      *)
(*  DAS EIGENWERTPROBLEM FR DIE RITZMATRIX ZU LSEN UND DAS LGS FR DIE    *)
(*  NEWTON-ITERATION                                                        *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

   REWRITE( AUS2 , 'ewt_ritz.aus' );

   WRITEPG( AUS );
   WRITE_HEAD( 'Ritz-Verfahren' );
   WRITELN( AUS , 'Startnherung des trigonometrischen Potentials :' );
   WRITELN( AUS );
   WRITELN( AUS , 'q(x) =   ' , FCP_AKT[0] );
   FOR I := 1 TO N DO BEGIN
       WRITE(   AUS , '       + ' , FCP_AKT[I] , ' cos( ' );
       WRITELN( AUS , 2 * I : 1 , 'x )' );
   END;

      
   COUNT := 0;
   
   REPEAT  (*  "ENDLOS"-SCHLEIFE  *)

      COUNT := COUNT + 1;
      
    (*EIGENWERTE UND FUNKTIONALMATRIX DES POTENTIALS FR 30 ANSATZFUNKTIONEN*)
    (*BEIM RITZ-VERFAHREN MIT DER PROZEDUR DEPT_RITZ IM MODUL DEPT BERECHNEN*)

      WRITELN( FUNKT_NAME );
      FOR I := 1 TO 8 DO BEGIN
          WRITE( 'Prozedur DEPT_RITZ berechnet Eigenwertnherungen bei der' );
          WRITELN( ' Ritz-Iteration...' , COUNT : 2 );
      END;

      DEPT_RITZ( FCP_AKT ,  N , 30 , EIG_AKT , FUNKT_MAT );

      (*  EIGENWERTE UND FUNKTIONALMATRIX AUSGEBEN  *)
       
      WRITEPG( AUS );
      WRITE_HEAD( 'Ritz-Verfahren' );
      IF COUNT = 1 THEN BEGIN
         WRITE( AUS , 'Eigenwerte der Startn' );
         WRITE( AUS2 , 'Funktionalmatrix der Startn' );
         END
      ELSE BEGIN
         WRITE( AUS , 'Eigenwerte der ' );
         WRITE( AUS , COUNT - 1 : 1 , '. N' );
         WRITE( AUS2 , 'Funktionalmatrix der ' );
         WRITE( AUS2 , COUNT - 1 : 1 , '. N' );
      END;
      WRITELN( AUS , 'herung :' );
      WRITELN( AUS );
      WRITELN( AUS2 , 'herung :' );
      WRITELN( AUS2 );
      FOR I := 0 TO N DO
          WRITELN( AUS , I : 1 , '. Eigenwert : ' , EIG_AKT[I] );
      WRITELN( AUS );
      FOR I := 1 TO N DO BEGIN
          WRITE(   AUS , I : 1 , '. Eigenwertdifferenz : ' );
          WRITELN( AUS , EIG_AKT[I] - EIG_AKT[I-1] );
      END;
      WRITELN( AUS );
      MRWRITE( AUS2 , 'DF' , FUNKT_MAT );

      (*  NCHSTE ITERIERTE DES POTENTIALS BERECHNEN  *)
      
      WRITELN( FUNKT_NAME );
      FOR I := 1 TO 8 DO BEGIN
          WRITE( 'Prozedur LSS lst Newton-Schritt bei der Ritz-Iteration.' );
          WRITELN( '..................' , COUNT : 2 );
      END;

      LSS( FUNKT_MAT , EIG_ZIEL - EIG_AKT , ZUWACHS , ERR );
      EIG_AKT := EIG_ZIEL;
      ZUWACHS_MID := MID( ZUWACHS );
      FCP_AKT := FCP_AKT + ZUWACHS_MID;

      (*  HILFSGRSSEN FR DEN EINSCHLIESSUNGSSCHRITT BERECHNEN  *)

      ETA_ALT := ETA;
      ETA := MAX_NORM( ZUWACHS_MID );
                              
      WRITELN( AUS );
      WRITELN( AUS , 'Zuwachs  q^(', COUNT:1, ') - q(', COUNT-1:1, ') : ' );
      WRITELN( AUS );
      WRITELN( AUS , 'ZW(x) =   ' , ZUWACHS_MID[0] );
      FOR I := 1 TO N DO BEGIN
          WRITE( AUS, '        + ', ZUWACHS_MID[I]-ZUWACHS_MID[0], ' cos( ' );
          WRITELN( AUS , 2 * I : 1 , 'x )' );
      END;
      WRITELN( AUS );
      WRITELN( AUS );
      WRITELN( AUS , 'eta_' , COUNT : 1 , ' = Maxnorm( Zuwachs ) : ' , ETA );
      WRITELN( AUS );
      WRITE(   AUS , '8 * eta_' , COUNT : 1 , '^3 / ' );
      WRITE(   AUS , '( Maxnorm( q^(' , COUNT : 1 , ') ) * eta_' );
      WRITE(   AUS , COUNT - 1 : 1 , '^2 ) : ' );
      WRITELN( AUS , 8 * ( ETA**3 ) / ( MAX_NORM(FCP_AKT) * SQR(ETA_ALT) ) );

      WRITEPG( AUS );
      WRITE_HEAD( 'Ritz-Verfahren' );
      WRITELN( AUS,COUNT:1 , '. Nherung des trigonometrischen Potentials :' );
      WRITELN( AUS );
      WRITELN( AUS , 'q(x) =   ' , FCP_AKT[0] );
      FOR I := 1 TO N DO BEGIN
          WRITE(   AUS , '       + ' , FCP_AKT[I] , ' cos( ' );
          WRITELN( AUS , 2 * I : 1 , 'x )' );
      END;

   UNTIL ( MAX_NORM( MID ( ZUWACHS ) ) < 1.0E-04 * MAX_NORM( FCP_AKT ) )
           OR ( COUNT = 10 );

   IF COUNT = 10 THEN BEGIN

      (*  ABBRUCH MIT VERMUTETER DIVERGENZ  *)

      WRITELN( AUS );
      WRITELN( AUS );
      WRITE(   AUS , 'Abbruch wegen berschreitung der Hchstzahl ' );
      WRITELN( AUS , 'an Iterationen.' );
      WRITELN( AUS );
      WRITELN( AUS , 'Das Verfahren scheint zu divergieren.' );
      WRITEPG( AUS );

      (*  SPRUNG ZUM PROGRAMMENDE  *)

      GOTO 99;

   END;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  SCHRITT 2 : BERECHNUNG EINES GUTEN NHERUNGSPOTENTIALS MIT DEM          *)
(*              VERFAHREN VON LOHNER                                        *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME NHERUNGSWEISE GELST                               *)
(*                                                                          *)
(*  AUSGANGSPROBLEM :      - U" + P(X)  U(X) =   U(X)                    *)
(*                                                                          *)
(*  TRANSFORMIERT AUF :      U1' = U2                                       *)
(*                           U2' = ( P(X) -  )  U1                        *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

   REWRITE( AUS2 , 'ewt_app.aus' );

   (*  BASISPOTENTIALE BERECHNEN  *)

   FOR I := 0 TO N DO BEGIN
       FOR J := 0 TO N DO
           FC_Q[I,J] := 0;
       FC_Q[I,I] := 1;
   END;

   (*  HEURISTISCH OPTIMALEN DIVISOR BERECHNEN        *)
   (*  DIVISOREN STRECKEN DAS ZEITINTERVALL DER DGL,  *)
   (*  STEUERN ALSO INDIREKT DIE SCHRITTWEITE         *)

   FOR EW := 0 TO N DO BEGIN
       HNORM := ABS( FCP_AKT[0] - EIG_ZIEL[EW] );
       FOR I := 1 TO N DO
           HNORM := HNORM + ABS( FCP_AKT[I] );
       HNORM := SQRT( HNORM );
       DIVISOR[EW] := 1 + ROUND( ( 6 * HNORM + 15 ) / M );

       IF DIVISOR[EW] * M > 700 THEN BEGIN

          (*  SPEICHERPLATZBEDINGTER ABBRUCH  *)

          WRITEPG( AUS );
          WRITE_HEAD( 'Anfangswertprobleme nherungsweise gelst' );
          WRITELN( AUS );
          WRITE(   AUS , 'Es sind zu viele Sttzstellen, nmlich ' );
          WRITELN( AUS , M * DIVISOR[EW] : 1 , ' statt 700.' );
          GOTO 99;

       END;

   END;

   WRITEPG( AUS );
   WRITE_HEAD( 'Anfangswertprobleme nherungsweise gelst' );
   WRITE(   AUS , 'Intervall I = [ 0 ,  ]        ' );
   WRITELN( AUS , 'Taylor-Ordnung P = ' , P : 1 );
   WRITELN( AUS );
   WRITELN( AUS , 'Schrittweiten :' );
   WRITELN( AUS );
   FOR EW := 0 TO N DO
       WRITELN( AUS , EW:1 , '. Eigenwert :  h =  / ' , M * DIVISOR[EW]:1 );
   WRITELN( AUS );
   WRITELN( AUS );
   WRITELN( AUS , 'Startnherung des trigonometrischen Potentials :' );
   WRITELN( AUS );
   WRITELN( AUS , 'q(x) =   ' , FCP_AKT[0] );
   FOR I := 1 TO N DO BEGIN
       WRITE(   AUS , '       + ' , FCP_AKT[I] , ' cos( ' );
       WRITELN( AUS , 2 * I : 1 , 'x )' );
   END;
      
   COUNT := 0;
   
   REPEAT  (*  "ENDLOS"-SCHLEIFE  *)

      COUNT := COUNT + 1;
      
      WRITEPG( AUS2 );
      WRITELN( AUS2 , 'Anfangswertprobleme nherungsweise gelst' );
      WRITELN( AUS2 );

      FOR EW := 0 TO N DO BEGIN

          DIVI   := DIVISOR[EW];
          DIVI_2 := SQR( DIVI );

          (*  TAYLORKOEFFIZIENTEN DER POTENTIALFUNKTION BEREITSTELLEN  *)

          IF ( EW = 0 ) OR ( DIVISOR[EW] <> DIVISOR[EW-1] ) THEN BEGIN
             
             FOR ST := 0 TO M * DIVI - 1 DO BEGIN
                 T := H * ST;
                 TRVECP[ST] := TAYKFCPT( T , FCP_AKT );
                 FOR J := 0 TO N DO
                     TRMATQ[ J, ST ] := DIVI_2 * TAYKFCPT( T , FC_Q[J] );
             END;

          END;

          (*  VORITERATION : FUNKTIONSWERTE AN DER STELLE "FCP" BERECHNEN  *)

          (*  PROZEDUR EW_APP IM MODUL DEP BERECHNET EIGENWERTNHERUNG  *)
          (*  BIS ZUR MAXIMAL DURCH LSEN VON ANFANGSWERTPROBLEMEN AUF  *)
          (*  DEM RECHNER ERZIELBAREN GENAUIGKEIT (=STILLSTAND DER EW-  *)
          (*  ITERATION DURCH DAS SEKANTENVERFAHREN)                    *)

          EIG_AKT[EW] := EW_APP( M, P, EW, DIVI, TRVECP, EIG_ZIEL[EW], AUS2 );

          (*  ZUR BERECHNUNG DER FUNKTIONALMATRIX  *)
          (*  SIND ANFANGSWERTPROBLEME ZU LSEN    *)

          AWA_NWT( M * DIVI , P , H , EIG_AKT[EW] / DIVI_2 ,
                   TRVECP , TRMATQ , VRU , VR_U2 , VR_U2Q );

          WRITELN( AUS2 , ' = ' , EIG_AKT[EW] , ' :' );
          WRITELN( AUS2 );
          WRITELN( AUS2 , 'U  ( ' , DIVI : 2 , ' ) = ' , VRU[1] );
          WRITELN( AUS2 , 'U'' ( ' , DIVI : 2 , ' ) = ' , VRU[2] );
          WRITELN( AUS2 , 'U( ' , DIVI : 2 , ' ) = ' , VR_U2 );

          FOR I := 0 TO N DO
              WRITELN( AUS2 , 'Q[' , I : 1 , '] * U = ' , VR_U2Q[I] );
          WRITELN( AUS2 );

          (*  EW-TE ZEILE DER FUNKTIONALMATRIX BERECHNEN                 *)

          FOR I := 0 TO N DO
              FUNKT_MAT[EW,I] := VR_U2Q[I] / VR_U2;

      END; (* FOR , EW *)

      (*  FUNKTIONALMATRIX AUSGEBEN  *)
       
      WRITEPG( AUS2 );
      WRITELN( AUS2 , 'Anfangswertprobleme nherungsweise gelst' );
      WRITELN( AUS2 );
      IF COUNT = 1 THEN 
         WRITE( AUS2 , 'Funktionalmatrix der Startn' )
      ELSE
         WRITE( AUS2 , 'Funktionalmatrix der ' , COUNT - 1 : 1 , '. N' );
      WRITELN( AUS2 , 'herung :' );
      WRITELN( AUS2 );
      MRWRITE( AUS2 , 'DF' , FUNKT_MAT );

      (*  NCHSTE ITERIERTE DES POTENTIALS BERECHNEN  *)
      
      WRITELN( FUNKT_NAME );
      FOR I := 1 TO 8 DO BEGIN
          WRITE( 'Prozedur LSS lst Newton-Schritt bei der DGL-Iteration..' );
          WRITELN( '..................' , COUNT : 2 );
      END;

      LSS( FUNKT_MAT , EIG_ZIEL - EIG_AKT , ZUWACHS , ERR );
      EIG_AKT := EIG_ZIEL;
      ZUWACHS_MID := MID( ZUWACHS );
      FCP_AKT := FCP_AKT + ZUWACHS_MID;

      (*  HILFSGRSSEN FR DEN EINSCHLIESSUNGSSCHRITT BERECHNEN  *)

      ETA_ALT := ETA;
      ETA := MAX_NORM( ZUWACHS_MID );

      WRITEPG( AUS );
      WRITE_HEAD( 'Anfangswertprobleme nherungsweise gelst' );
      WRITELN( AUS , 'Zuwachs  q^(', COUNT:1, ') - q(', COUNT-1:1, ') : ' );
      WRITELN( AUS );
      WRITELN( AUS , 'ZW(x) =   ' , ZUWACHS_MID[0] );
      FOR I := 1 TO N DO BEGIN
          WRITE( AUS, '        + ', ZUWACHS_MID[I]-ZUWACHS_MID[0], ' cos( ' );
          WRITELN( AUS , 2 * I : 1 , 'x )' );
      END;
      WRITELN( AUS );
      WRITELN( AUS );
      WRITELN( AUS , 'eta_' , COUNT : 1 , ' = Maxnorm( Zuwachs ) : ' , ETA );
      WRITELN( AUS );
      WRITE(   AUS , '8 * eta_' , COUNT : 1 , '^3 / ' );
      WRITE(   AUS , '( Maxnorm( q^(' , COUNT : 1 , ') ) * eta_' );
      WRITE(   AUS , COUNT - 1 : 1 , '^2 ) : ' );
      WRITELN( AUS , 8 * ( ETA**3 ) / ( MAX_NORM(FCP_AKT) * SQR(ETA_ALT) ) );

      WRITEPG( AUS );
      WRITE_HEAD( 'Anfangswertprobleme nherungsweise gelst' );
      WRITELN( AUS,COUNT:1 , '. Nherung des trigonometrischen Potentials :' );
      WRITELN( AUS );
      WRITELN( AUS , 'q(x) =   ' , FCP_AKT[0] );
      FOR I := 1 TO N DO BEGIN
          WRITE(   AUS , '       + ' , FCP_AKT[I] , ' cos( ' );
          WRITELN( AUS , 2 * I : 1 , 'x )' );
      END;


(* UNTIL MAX_NORM( MID ( ZUWACHS ) ) < 1.0E-12 * MAX_NORM( FCP_AKT ); *)
   UNTIL 8 * ( ETA ** 3 ) / ( MAX_NORM( FCP_AKT ) * SQR( ETA_ALT ) ) < 1.0E-14;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  SCHRITT 3 : BERECHNUNG EINES EINSCHLIESSENDEN INTERVALLPOTENTIALS       *)
(*              MIT DEM VERFAHREN VON LOHNER                                *)
(*                                                                          *)
(*  ANFANGSWERTPROBLEME MIT INTERVALLEINSCHLIESSUNG GELST                  *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

   REWRITE( AUS2 , 'ewt_bds.aus' );

   FOR I := 0 TO N DO
       IFCP_MID[I] := FCP_AKT[I];

   (*  LETZTE FUNKTIONALMATRIX BESETZEN  *)

   FOR J := 0 TO N DO
       FOR I := 0 TO N DO
           ILAST_FUNKT_MAT[I,J] := FUNKT_MAT[I,J];

   (*  BASISPOTENTIALE BERECHNEN  *)

   FOR I := 0 TO N DO BEGIN
       FOR J := 0 TO N DO
           IFC_Q[I,J] := 0;
       IFC_Q[I,I] := 1;
   END;

   WRITEPG( AUS );
   WRITE_HEAD( 'Einschlieungsschritt' );
   WRITE(   AUS , 'Intervall I = [ 0 ,  ]        ' );
   WRITELN( AUS , 'Taylor-Ordnung P = ' , P : 1 );
   WRITELN( AUS );
   WRITELN( AUS , 'Schrittweiten :' );
   WRITELN( AUS );
   FOR EW := 0 TO N DO
       WRITELN( AUS , EW:1 , '. Eigenwert :  h =  / ' , M * DIVISOR[EW]:1 );
   WRITELN( AUS );
   WRITELN( AUS );
      
   COUNT := 0;
   
   REPEAT  (*  "ENDLOS"-SCHLEIFE  *)

      COUNT := COUNT + 1;
      
      FOR EW := 0 TO N DO BEGIN

          DIVI   := DIVISOR[EW];
          DIVI_2 := SQR( DIVI );

          IF DIVI * M > 700 THEN BEGIN

             (*  SPEICHERPLATZBEDINGTER ABBRUCH  *)

             WRITEPG( AUS );
             WRITE_HEAD( 'Anfangswertprobleme intervallmig gelst' );
             WRITELN( AUS );
             WRITE(   AUS , 'Es sind zu viele Sttzstellen, nmlich ' );
             WRITELN( AUS , M * DIVI : 1 , ' statt 700.' );
             GOTO 99;

          END;

          (*  TAYLORKOEFFIZIENTEN DER POTENTIALFUNKTION BEREITSTELLEN  *)

          IF ( EW = 0 ) OR ( DIVISOR[EW] <> DIVISOR[EW-1] ) THEN BEGIN
             
             FOR ST := 0 TO M * DIVI - 1 DO BEGIN
                 IT := IH * ST;
                 TIVECP_MID[ST] := TAYKFCPT( IT , IFCP_MID );
                 TIVECP_RST_MID[ST] := TAYKFCPT( IT + IHI , IFCP_MID );
             END;

          END;

          (*  FUNKTIONSWERTE AN DER STELLE "IFCP" BERECHNEN  *)

          (*  PROZEDUR  EW_BDS  IM MODUL DEP BERECHNET EIGENWERTSCHRANKEN  *)
          (*  GENAUIGKEITS-PARAMETER 0.98 = MAXIMALE GENAUIGKEIT           *)

          IEIG_AKT_MID[EW] := EIG_ZIEL[EW];
          
          IEIG_AKT_MID[EW] := EW_BDS( M , P , EW , DIVI , TIVECP_MID ,
                              TIVECP_RST_MID , IEIG_AKT_MID[EW], 0.98, AUS2 );

      END; (* FOR , EW *)
      
      (*  JETZT INTERVALLPOTENTIAL GENERIEREN, IN DEM DIE LSUNG VERMUTET  *)
      (*  WIRD; VERFAHREN VON ALEFELD-GIENGER-POTRA VERWENDEN              *)

      (*  ZUM VERGLEICH ZUERST INNERE MINDESTEINSCHLIESSUNG BERECHNEN !!!  *)

      WRITELN( FUNKT_NAME );
      FOR I := 1 TO 8 DO BEGIN
          WRITE( 'Prozedur LSS berechnet Mindesteinschlieung.............' );
          WRITELN( '..................' , COUNT : 2 );
      END;

      LSS( ILAST_FUNKT_MAT , IEIG_ZIEL - IEIG_AKT_MID , ZUWACHS , ERR );

      WIDTH_AKT := ZUWACHS[0].SUP -> ZUWACHS[0].INF;
    
      FOR I := 1 TO N DO IF WIDTH_AKT <  ZUWACHS[I].SUP -> ZUWACHS[I].INF THEN
                            WIDTH_AKT := ZUWACHS[I].SUP -> ZUWACHS[I].INF;

      WRITELN( AUS , 'Durchmesser der Mindesteinschlieung : ' , WIDTH_AKT );
      WRITELN( AUS );
      WRITELN( AUS , 'Durchmesser der AGP-Einschlieung    : ' , 2 * ETA );
      AGP := TRUE;
      IF WIDTH_AKT > 2 * ETA THEN BEGIN
         WRITELN( AUS );
         WRITE(   AUS , 'Der Durchmesser der AGP-Einschlieung ' );
         WRITELN( AUS , 'ist zu klein!' );
         WRITELN( AUS );
         WRITE(   AUS , 'Die AGP-Einschlieung wird mit (eta_k * eta_k-1)' );
         WRITELN( AUS , ' durchgefhrt.' );
         ETA := SQRT( ETA * ETA_ALT );
         WRITELN( AUS );
         WRITE(   AUS , 'Durchmesser der vergrberten AGP-Einschlieung : ' );
         WRITELN( AUS , 2 * ETA );
         IF WIDTH_AKT > 2 * ETA THEN BEGIN
            WRITELN( AUS );
            WRITE( AUS,'Der Durchmesser der vergrberten AGP-Einschlieung ');
            WRITELN( AUS , 'ist auch zu klein!' );
            WRITELN( AUS );
            WRITE(   AUS , 'Die Einschlieung wird mit hunderfacher ' );
            WRITELN( AUS , 'Mindesteinschlieung durchgefhrt.' );
            AGP := FALSE;
            ETA := 50 * WIDTH_AKT;
            WRITELN( AUS );
            WRITE(AUS,'Durchmesser der hundertfachen Mindesteinschlieung : ');
            WRITELN( AUS , 2 * ETA );
         END;
      END
      ELSE IF WIDTH_AKT > 0.4 * ETA THEN BEGIN (*  AGP <= 5 * MINDEST  *)
         WRITELN( AUS );
         WRITE(   AUS , 'Die AGP-Einschlieung ist nur ' );
         WRITELN( AUS , 'wenig grer als die Mindesteinschlieung' );
         WRITELN( AUS );
         WRITE(   AUS , 'Die AGP-Einschlieung wird mit (eta_k * eta_k-1)' );
         WRITELN( AUS , ' durchgefhrt.' );
         ETA := SQRT( ETA * ETA_ALT );
         WRITELN( AUS );
         WRITE(   AUS , 'Durchmesser der vergrberten AGP-Einschlieung : ' );
         WRITELN( AUS , 2 * ETA );
      END;

      (*  JETZT INTERVALLPOTENTIAL GENERIEREN, IN DEM DIE LSUNG        *)
      (*  VERMUTET WIRD                                                 *)
      (*  BEI DER EINSCHLIESSUNG DER EIGENFUNKTIONEN IST DIE KONSTANTE  *)
      (*  BASISFUNKTION q_0(x) = 1 UNERHEBLICH!                         *)

      IFCP_AKT[0] := IFCP_MID[0];
      FOR I := 1 TO N DO
          IFCP_AKT[I] := IFCP_MID[I] + ETA * INTVAL( -1 , 1 );

      WIDTH_AKT := 2 * ETA;

      WRITEPG( AUS );
      IF AGP THEN
         WRITE_HEAD( 'AGP-Einschlieungsschritt' )
      ELSE
   WRITE_HEAD('Einschlieungsschritt mit hundertfacher Mindesteinschlieung');
      WRITELN( AUS , 'Start-Intervalleinschlieung :' );
      WRITELN( AUS );
      WRITE( AUS , 'q(x) =   ' );
      IWRITELN( AUS , IFCP_AKT[0] + ETA * INTVAL( -1 , 1 ) );
      FOR I := 1 TO N DO BEGIN
          WRITE(   AUS , '       + ' );
          IWRITE(  AUS , IFCP_AKT[I] );
          WRITELN( AUS , ' cos( ' , 2 * I : 1 , 'x )' );
      END;

      WRITEPG( AUS );
      IF AGP THEN
         WRITE_HEAD( 'AGP-Einschlieungsschritt' )
      ELSE
   WRITE_HEAD('Einschlieungsschritt mit hundertfacher Mindesteinschlieung');
      WRITELN( AUS2 );
      IF AGP THEN
         WRITELN( AUS2 , 'AGP-Einschlieungsschritt' )
      ELSE
WRITELN(AUS2,'Einschlieungsschritt mit hundertfacher Mindesteinschlieung');
      WRITELN( AUS2 );

      (*  EIGENWERTSCHRANKEN DES INTERVALLPOTENTIALS MIT DEM MONOTONIE-  *)
      (*  PRINZIP AUS DEN SCHRANKEN DES MITTELPUNKTPOTENTIALS BERECHNEN  *)

      (* MAXIMALEN DURCHMESSER DURCH MAXIMAL MGLICHEN DURCHMESSER ABSCHTZEN*)
      (* ETA JE 1-MAL IN FCP[1..N] *)

      WIDTH_H := N * ETA;

      (*  FUNKTIONALMATRIX BERECHNEN  *)
      
      FOR EW := 0 TO N DO BEGIN

          IEIG_AKT[EW] := IEIG_AKT_MID[EW] + INTVAL( -WIDTH_H , WIDTH_H );

          DIVI   := DIVISOR[EW];
          DIVI_2 := SQR( DIVI );

          (*  TAYLORKOEFFIZIENTEN DER POTENTIALFUNKTION BEREITSTELLEN  *)

          IF ( EW = 0 ) OR ( DIVISOR[EW] <> DIVISOR[EW-1] ) THEN BEGIN
             
             FOR ST := 0 TO M * DIVI - 1 DO BEGIN
                 IT := IH * ST;
                 TIVECP[ST] := TAYKFCPT( IT , IFCP_AKT );
                 TIVECP_RST[ST] := TAYKFCPT( IT+IHI , IFCP_AKT );
                 FOR J := 0 TO N DO BEGIN
                    TIMATQ[ J, ST ] := DIVI_2 * TAYKFCPT( IT , IFC_Q[J] );
                    TIMATQ[ J, ST, P ] := RESTGLIED( IT + IHI , IFC_Q[J] );
                 END;
             END;

          END;

          (*  ZUR BERECHNUNG DER FUNKTIONALMATRIX  *)
          (*  SIND ANFANGSWERTPROBLEME ZU LSEN    *)

          AWA_NWT( M * DIVI , P , IH , IEIG_AKT[EW] / DIVI_2 ,
                   TIVECP , TIVECP_RST , TIMATQ , VIU , VI_U2 , VI_U2Q );

          WRITELN(  AUS , 'Einschlieungen des ' , EW : 1 , '. Eigenwerts :' );
          WRITELN(  AUS );
          WRITE(    AUS , 'Mittelpunktsnherung : ' );
          IWRITELN( AUS , IEIG_AKT_MID[EW] );
          WRITE(    AUS , 'Intervallspline      : ' );
          IWRITELN( AUS , IEIG_AKT[EW] );
          WRITELN(  AUS );
          WRITE(    AUS2 , 'Werte der ' , EW : 1 , '. "Eigenfunktion" '  );
          WRITELN(  AUS2 , 'des Intervallsplines :'  );
          WRITELN(  AUS2 );
          WRITE(    AUS2 , 'U  ( ' , DIVI : 2 , ' ) = ' );
          IWRITELN( AUS2 , VIU[1] );
          WRITE(    AUS2 , 'U'' ( ' , DIVI : 2 , ' ) = ' );
          IWRITELN( AUS2 , VIU[2] );
          WRITE(    AUS2 , ' U       = ' );
          IWRITELN( AUS2 , VI_U2);

          FOR I := 0 TO N DO BEGIN
              WRITE( AUS2 , 'Q[' , I : 1 , '] * U = ' );
              IWRITELN( AUS2 , VI_U2Q[I] );
          END; 
          WRITELN( AUS2 );

          (*  EW-TE ZEILE DER FUNKTIONALMATRIX BERECHNEN                 *)

          FOR I := 0 TO N DO
              IFUNKT_MAT[EW,I] := VI_U2Q[I] / VI_U2;

      END; (* FOR , EW *)

      (*  FUNKTIONALMATRIX AUSGEBEN  *)
       
      WRITEPG( AUS2 );
      WRITELN( AUS2 );
      IF AGP THEN
         WRITELN( AUS2 , 'AGP-Einschlieungsschritt' )
      ELSE
 WRITELN(AUS2,'Einschlieungsschritt mit hundertfacher Mindesteinschlieung');
      WRITELN( AUS2 );
      IF COUNT = 1 THEN 
         WRITE( AUS2 , 'Funktionalmatrix der Startn' )
      ELSE
         WRITE( AUS2 , 'Funktionalmatrix der ' , COUNT - 1 : 1 , '. N' );
      WRITELN( AUS2 , 'herung :' );
      WRITELN( AUS2 );
      MIWRITE( AUS2 , 'DF' , IFUNKT_MAT );
      
      (*  NCHSTE ITERIERTE DES POTENTIALS BERECHNEN  *)
      
      WRITELN( FUNKT_NAME );
      FOR I := 1 TO 8 DO BEGIN
          WRITE( 'Prozedur LSS lst Newton-Schritt bei der Verifizierung' );
          WRITELN( '....................' , COUNT : 2 );
      END;

      LSS( IFUNKT_MAT , IEIG_ZIEL - IEIG_AKT_MID , ZUWACHS , ERR );

      (*  ZUERST NOCH IFCP_AKT[0] EBENFALLS AUFBLHEN  *)

      IFCP_AKT[0] := IFCP_MID[0] + ETA * INTVAL( -1 , 1 );
      IFCP_ALT := IFCP_AKT;
      IFCP_AKT := IFCP_MID + ZUWACHS;
           
      WRITEPG( AUS );
      IF AGP THEN
         WRITE_HEAD( 'AGP-Einschlieungsschritt' )
      ELSE
   WRITE_HEAD('Einschlieungsschritt mit hundertfacher Mindesteinschlieung');
      WRITELN( AUS , COUNT : 1 , '. Nherung des trigonometrischen Potentials :' );
      WRITELN( AUS );
      WRITE(   AUS , 'q(x) =   '  );
      IWRITELN( AUS , IFCP_AKT[0] );
      FOR I := 1 TO N DO BEGIN
          WRITE(   AUS , '       + ' );
          IWRITE(  AUS , IFCP_AKT[I] );
          WRITELN( AUS , ' cos( ' , 2 * I : 1 , 'x )' );
      END;

      WIDTH_ALT := 0;
      WIDTH_AKT := 0;
      NCL := TRUE;
      NCL_MID := TRUE;
    
      FOR I := 0 TO N DO BEGIN
     
         IF ( IFCP_ALT[I].SUP -> IFCP_ALT[I].INF > WIDTH_ALT ) THEN
            WIDTH_ALT := IFCP_ALT[I].SUP -> IFCP_ALT[I].INF;

         IF ( IFCP_AKT[I].SUP -> IFCP_AKT[I].INF > WIDTH_AKT ) THEN
            WIDTH_AKT := IFCP_AKT[I].SUP -> IFCP_AKT[I].INF;

          IF NOT ( IFCP_AKT[I] IN IFCP_ALT[I] ) THEN
             NCL := FALSE;

          IF NOT ( IFCP_MID[I] IN IFCP_AKT[I] ) THEN
             NCL_MID := FALSE;

      END;
   
      WRITELN( AUS );
      WRITELN( AUS );
      WRITELN( AUS );
      IF AGP THEN
         WRITE( AUS , 'Eine verifizierte AGP-Einschlieung konnte ' )
      ELSE
         WRITE( AUS , 'Eine verifizierte Einschlieung konnte ' );
      IF NCL THEN
         WRITELN( AUS , 'berechnet werden.' )
      ELSE
         WRITELN( AUS , 'nicht berechnet werden.' );
      WRITELN( AUS );
      WRITE( AUS , 'Die alte (Punkt-) Nherungslsung ist ' );
      IF NCL_MID THEN
         WRITELN( AUS , 'noch in der neuen Einschlieung enthalten.' )
      ELSE BEGIN
         WRITELN( AUS , 'in der neuen Einschlieung nicht mehr' );
      WRITELN( AUS , 'enthalten.' );
      END;
      WRITELN( AUS );
      WRITELN( AUS , 'Maximaler Durchmesser von [q(x)]_ALT : ' , WIDTH_ALT );
      WRITELN( AUS );
      WRITELN( AUS , 'Eigenwert-Durchmesser von [q(x)]_ALT : ' , 2 * WIDTH_H );
      WRITELN( AUS );
      WRITELN( AUS , 'Maximaler Durchmesser von [q(x)]_NEU : ' , WIDTH_AKT );

(* UNTIL NCL;*)
   UNTIL TRUE;    (*  DERZEITIG ABBRECHEN NACH EINEM EINSCHLIESSUNGSVERSUCH *)

   WRITEPG( AUS );

99: END.

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - PROGRAMM  IEPT_AGP                                         *)
(*                                                                          *)
(*  NEWTON-VERFAHREN ZUR BERECHNUNG EINES SYMMETRISCHEN TRIGONOMETRISCHEN   *)
(*  POTENTIALS ZU VORGEGEBENEN EIGENWERTEN                                  *)
(*                                                                          *)
(*  VOLLSTNDIGES REKONSTRUKTIONSVERFAHREN AUS DER DISSERTATION             *)
(*  EINSCHLIESSUNGSSCHRITT STARTET MIT ALEFELD-GIENGER-POTRA-NHERUNG       *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)
